package com.sun.jmx.snmp.agent;

import com.sun.jmx.snmp.SnmpOid;
import java.util.Vector;
import java.util.Enumeration;
import com.sun.jmx.snmp.SnmpStatusException;
import com.sun.jmx.snmp.SnmpVarBind;
import java.io.Serializable;

public class SnmpMibOid extends SnmpMibNode implements Serializable
{
    private static final long serialVersionUID = 5012254771107446812L;
    private SnmpMibOid.NonSyncVector<SnmpMibNode> children;
    private int nbChildren;
    
    public SnmpMibOid() {
        this.children = new SnmpMibOid.NonSyncVector(1);
        this.nbChildren = 0;
    }
    
    @Override
    public void get(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        Enumeration<SnmpVarBind> elements = snmpMibSubRequest.getElements();
        while (elements.hasMoreElements()) {
            snmpMibSubRequest.registerGetException((SnmpVarBind)elements.nextElement(), new SnmpStatusException(225));
        }
    }
    
    @Override
    public void set(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        Enumeration<SnmpVarBind> elements = snmpMibSubRequest.getElements();
        while (elements.hasMoreElements()) {
            snmpMibSubRequest.registerSetException((SnmpVarBind)elements.nextElement(), new SnmpStatusException(6));
        }
    }
    
    @Override
    public void check(SnmpMibSubRequest snmpMibSubRequest, int n) throws SnmpStatusException {
        Enumeration<SnmpVarBind> elements = snmpMibSubRequest.getElements();
        while (elements.hasMoreElements()) {
            snmpMibSubRequest.registerCheckException((SnmpVarBind)elements.nextElement(), new SnmpStatusException(6));
        }
    }
    
    @Override
    void findHandlingNode(SnmpVarBind snmpVarBind, long[] array, int n, SnmpRequestTree snmpRequestTree) throws SnmpStatusException {
        int length = array.length;
        if (snmpRequestTree == null) {
            throw new SnmpStatusException(5);
        }
        if (n > length) {
            throw new SnmpStatusException(225);
        }
        if (n == length) {
            throw new SnmpStatusException(224);
        }
        SnmpMibNode child = this.getChild(array[n]);
        if (child == null) {
            snmpRequestTree.add(this, n, snmpVarBind);
        }
        else {
            child.findHandlingNode(snmpVarBind, array, n + 1, snmpRequestTree);
        }
        return;
    }
    
    @Override
    long[] findNextHandlingNode(SnmpVarBind snmpVarBind, long[] array, int n, int n2, SnmpRequestTree snmpRequestTree, AcmChecker acmChecker) throws SnmpStatusException {
        int length = array.length;
        long[] nextHandlingNode = null;
        if (snmpRequestTree == null) {
            throw new SnmpStatusException(225);
        }
        Object userData = snmpRequestTree.getUserData();
        int requestPduVersion = snmpRequestTree.getRequestPduVersion();
        if (n >= length) {
            return this.findNextHandlingNode(snmpVarBind, new long[] { this.getNextVarId(-1L, userData, requestPduVersion) }, 0, n2, snmpRequestTree, acmChecker);
        }
        long[] array2 = { 0L };
        long nextVarId = array[n];
        Label_0095: {
            try {
                SnmpMibNode child = this.getChild(nextVarId);
                if (child == null) {
                    throw new SnmpStatusException(225);
                }
                acmChecker.add(n2, nextVarId);
                try {
                    nextHandlingNode = child.findNextHandlingNode(snmpVarBind, array, n + 1, n2 + 1, snmpRequestTree, acmChecker);
                }
                finally {
                    acmChecker.remove(n2);
                }
                nextHandlingNode[n2] = nextVarId;
                return nextHandlingNode;
            }
            catch (SnmpStatusException ex) {
                nextVarId = this.getNextVarId(nextVarId, userData, requestPduVersion);
                array2[0] = nextVarId;
                n = 1;
                array = array2;
                continue;
            }
        }
    }
    
    @Override
    public void getRootOid(Vector<Integer> vector) {
        if (this.nbChildren != 1) {
            return;
        }
        vector.addElement(Integer.valueOf(this.varList[0]));
        ((SnmpMibNode)this.children.firstElement()).getRootOid(vector);
        return;
    }
    
    public void registerNode(String s, SnmpMibNode snmpMibNode) throws IllegalAccessException {
        this.registerNode(new SnmpOid(s).longValue(), 0, snmpMibNode);
    }
    
    void registerNode(long[] array, int n, SnmpMibNode obj) throws IllegalAccessException {
        if (n >= array.length) {
            throw new IllegalAccessException();
        }
        long n2 = array[n];
        int retrieveIndex = this.retrieveIndex(n2);
        if (retrieveIndex == this.nbChildren) {
            ++this.nbChildren;
            (this.varList = new int[this.nbChildren])[0] = (int)n2;
            int n3 = 0;
            if (n + 1 == array.length) {
                this.children.insertElementAt(obj, n3);
                return;
            }
            SnmpMibOid obj3 = new SnmpMibOid();
            this.children.insertElementAt(obj3, n3);
            obj3.registerNode(array, n + 1, obj);
            return;
        }
        else {
            if (retrieveIndex == -1) {
                int[] varList = new int[this.nbChildren + 1];
                varList[this.nbChildren] = (int)n2;
                System.arraycopy(this.varList, 0, varList, 0, this.nbChildren);
                this.varList = varList;
                ++this.nbChildren;
                SnmpMibNode.sort(this.varList);
                int retrieveIndex2 = this.retrieveIndex(n2);
                this.varList[retrieveIndex2] = (int)n2;
                if (n + 1 == array.length) {
                    this.children.insertElementAt(obj, retrieveIndex2);
                    return;
                }
                SnmpMibOid obj2 = new SnmpMibOid();
                this.children.insertElementAt(obj2, retrieveIndex2);
                obj2.registerNode(array, n + 1, obj);
            }
            else {
                SnmpMibNode snmpMibNode = (SnmpMibNode)this.children.elementAt(retrieveIndex);
                if (n + 1 == array.length) {
                    if (snmpMibNode == obj) {
                        return;
                    }
                    if (snmpMibNode != null && obj != null) {
                        if ((obj instanceof SnmpMibGroup)) {
                            ((SnmpMibOid)snmpMibNode).exportChildren((SnmpMibOid)obj);
                            this.children.setElementAt(obj, retrieveIndex);
                            return;
                        }
                        if (obj instanceof SnmpMibOid && snmpMibNode instanceof SnmpMibGroup) {
                            ((SnmpMibOid)obj).exportChildren((SnmpMibOid)snmpMibNode);
                            return;
                        }
                        if ((obj instanceof SnmpMibOid)) {
                            ((SnmpMibOid)snmpMibNode).exportChildren((SnmpMibOid)obj);
                            this.children.setElementAt(obj, retrieveIndex);
                            return;
                        }
                    }
                    this.children.setElementAt(obj, retrieveIndex);
                }
                else {
                    if (snmpMibNode == null) {
                        throw new IllegalAccessException();
                    }
                    ((SnmpMibOid)snmpMibNode).registerNode(array, n + 1, obj);
                }
            }
            return;
        }
    }
    
    void exportChildren(SnmpMibOid snmpMibOid) throws IllegalAccessException {
        if (snmpMibOid == null) {
            return;
        }
        long[] array = { 0L };
        for (int i = 0; i < this.nbChildren; ++i) {
            SnmpMibNode snmpMibNode = (SnmpMibNode)this.children.elementAt(i);
            if (snmpMibNode != null) {
                array[0] = (long)this.varList[i];
                snmpMibOid.registerNode(array, 0, snmpMibNode);
            }
        }
        return;
    }
    
    SnmpMibNode getChild(long n) throws SnmpStatusException {
        int insert = this.getInsertAt(n);
        if (insert >= this.nbChildren) {
            throw new SnmpStatusException(225);
        }
        if (this.varList[insert] != (int)n) {
            throw new SnmpStatusException(225);
        }
        SnmpMibNode snmpMibNode;
        try {
            snmpMibNode = (SnmpMibNode)this.children.elementAtNonSync(insert);
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            throw new SnmpStatusException(225);
        }
        if (snmpMibNode == null) {
            throw new SnmpStatusException(224);
        }
        return snmpMibNode;
    }
    
    private int retrieveIndex(long n) {
        int i = 0;
        int n2 = (int)n;
        if (this.varList == null || this.varList.length < 1) {
            return this.nbChildren;
        }
        int n3 = this.varList.length - 1;
        int n4 = i + (n3 - i) / 2;
        while (i <= n3) {
            int n5 = this.varList[n4];
            if (n2 == n5) {
                return n4;
            }
            if (n5 < n2) {
                i = n4 + 1;
            }
            else {
                n3 = n4 - 1;
            }
            n4 = i + (n3 - i) / 2;
        }
        return -1;
    }
    
    private int getInsertAt(long n) {
        int i = 0;
        int n2 = (int)n;
        if (this.varList == null) {
            return -1;
        }
        int n3 = this.varList.length - 1;
        int n4 = i + (n3 - i) / 2;
        while (i <= n3) {
            int n5 = this.varList[n4];
            if (n2 == n5) {
                return n4;
            }
            if (n5 < n2) {
                i = n4 + 1;
            }
            else {
                n3 = n4 - 1;
            }
            n4 = i + (n3 - i) / 2;
        }
        return n4;
    }
    
    class NonSyncVector<E> extends Vector<E>
    {
        public NonSyncVector(int initialCapacity) {
            super(initialCapacity);
        }
        
        final void addNonSyncElement(E e) {
            this.ensureCapacity(this.elementCount + 1);
            this.elementData[this.elementCount++] = e;
        }
        
        final E elementAtNonSync(int n) {
            return this.elementData[n];
        }
    }
}
